ArkInventoryRules_MoreRules = LibStub( "AceAddon-3.0" ):NewAddon( "ArkInventoryRules_MoreRules" )

function ArkInventoryRules_MoreRules:OnEnable( )
	
	ArkInventoryRules.Register( self, "CLASSGEAR",	ArkInventoryRules_MoreRules.ClassGear ) -- Gear that is usable by the current class
	ArkInventoryRules.Register( self, "CG", 		ArkInventoryRules_MoreRules.ClassGear ) -- Shorthand of ClassGear
	ArkInventoryRules.Register( self, "COLLECTED",	ArkInventoryRules_MoreRules.Collected ) -- If the item appearance is known by the player
	ArkInventoryRules.Register( self, "TMOG",		ArkInventoryRules_MoreRules.TMog ) -- If the item is even able to be transmogged
	ArkInventoryRules.Register( self, "UPGRADE",	ArkInventoryRules_MoreRules.Upgrade ) -- Gear that is of a higher ilvl than what is worn
	
end

local InventorySlots = {
    ['INVTYPE_HEAD'] = 1,
    ['INVTYPE_NECK'] = 2,
    ['INVTYPE_SHOULDER'] = 3,
    ['INVTYPE_BODY'] = 4,
    ['INVTYPE_CHEST'] = 5,
    ['INVTYPE_ROBE'] = 5,
    ['INVTYPE_WAIST'] = 6,
    ['INVTYPE_LEGS'] = 7,
    ['INVTYPE_FEET'] = 8,
    ['INVTYPE_WRIST'] = 9,
    ['INVTYPE_HAND'] = 10,
    ['INVTYPE_CLOAK'] = 15,
    ['INVTYPE_WEAPON'] = 16,
    ['INVTYPE_SHIELD'] = 17,
    ['INVTYPE_2HWEAPON'] = 16,
    ['INVTYPE_WEAPONMAINHAND'] = 16,
    ['INVTYPE_RANGED'] = 16,
    ['INVTYPE_RANGEDRIGHT'] = 16,
    ['INVTYPE_WEAPONOFFHAND'] = 17,
    ['INVTYPE_HOLDABLE'] = 17,
    -- ['INVTYPE_TABARD'] = 19,
}
--local EquipSlots = {
--    ['INVTYPE_HEAD'] = "HeadSlot",
--    ['INVTYPE_NECK'] = "NeckSlot",
--    ['INVTYPE_SHOULDER'] = "ShoulderSlot",
--    ['INVTYPE_CHEST'] = "ChestSlot",
--    ['INVTYPE_WAIST'] = "WaistSlot",
--    ['INVTYPE_LEGS'] = "LegsSlot",
--    ['INVTYPE_FEET'] = "FeetSlot",
--    ['INVTYPE_WRIST'] = "WristSlot",
--    ['INVTYPE_HAND'] = "HandsSlot",
--    ['INVTYPE_CLOAK'] = "BackSlot",
--    ['INVTYPE_WEAPON'] = "MainHandSlot",
--    ['INVTYPE_SHIELD'] = "SecondaryHandSlot",
--    ['INVTYPE_2HWEAPON'] = "MainHandSlot",
--    ['INVTYPE_WEAPONMAINHAND'] = "MainHandSlot",
--    ['INVTYPE_RANGED'] = "MainHandSlot",
--    ['INVTYPE_RANGEDRIGHT'] = "MainHandSlot",
--    ['INVTYPE_WEAPONOFFHAND'] = "SecondaryHandSlot",
--    ['INVTYPE_HOLDABLE'] = "SecondaryHandSlot",
    -- ['INVTYPE_TABARD'] = 19,
--}
 
local model = CreateFrame('DressUpModel')
function ArkInventoryRules_MoreRules.GetItemAppearance(itemLink)
    local itemID, _, _, slotName = GetItemInfoInstant(itemLink)
    if itemLink == itemID then
        itemLink = 'item:' .. itemID
    end
    local slot = InventorySlots[slotName]
    if not slot or not IsDressableItem(itemLink) then return end
    model:SetUnit('player')
    model:Undress()
    model:TryOn(itemLink, slot)
    local sourceID = model:GetSlotTransmogSources(slot)
    if sourceID then
        local categoryID, itemAppearanceID, canEnchant, texture, isCollected, itemLink = C_TransmogCollection.GetAppearanceSourceInfo(sourceID)
        return itemAppearanceID, isCollected, sourceID
    end
end
 
function ArkInventoryRules_MoreRules.PlayerHasAppearance(itemAppearanceID)
    local sources = C_TransmogCollection.GetAppearanceSources(itemAppearanceID)
    if sources then
        for i, source in pairs(sources) do
            if source.isCollected then
                return true
            end
        end
    end
    return false
end
 
function ArkInventoryRules_MoreRules.PlayerCanCollectAppearance(itemAppearanceID)
    local sources = C_TransmogCollection.GetAppearanceSources(itemAppearanceID)
    if sources then
        for i, source in pairs(sources) do
            if C_TransmogCollection.PlayerCanCollectSource(source.sourceID) then
                return true
            end
        end
    end
    return false
end
 
function ArkInventoryRules_MoreRules.CanTransmogItem(itemLink)
    local itemID = GetItemInfoInstant(itemLink)
    if itemID then
        local canBeChanged, noChangeReason, canBeSource, noSourceReason = C_Transmog.GetItemInfo(itemID)
        -- return canBeSource, noSourceReason
        if canBeSource then return true end
    end
    return false
end

function ArkInventoryRules_MoreRules.TMog( ... )
	local fn = "TMOG" -- Rule name for errors

		-- Verify that the item string information is loaded and not nil and that it is a valid item before continuing
	if select( 15, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) == nil or not ArkInventoryRules.Object.h or ArkInventoryRules.Object.class ~= "item" then
		return false
	end

	local itemID = select(2,ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) )
	local itemAppearanceID, isCollected, sourceID = ArkInventoryRules_MoreRules.GetItemAppearance(itemID)
	local objequip = 	select( 11, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) 
	
	if objequip == "INVTYPE_TRINKET" then return false end
	if objequip == "INVTYPE_NECK" then return false end
	if objequip == "INVTYPE_FINGER" then return false end
	if objequip == nil then return false end
	
	local quality = select(3, GetItemInfo(itemID))
	if quality == nil then return false end
	if quality <= 1 then return false end


	if not ArkInventoryRules_MoreRules.CanTransmogItem(itemID) then
		--ArkInventory.Output(string.lower( select( 3, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) ) ..  " NOT TRANSMOGABLE" )
		return false
	else
		--ArkInventory.Output(string.lower( select( 3, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) ) ..  " TRANSMOGABLE" )
		return true
	end

-- Former dependency on CanIMogIt
	-- Is CanIMogIt loaded?
--	if IsAddOnLoaded( "CanIMogIt" ) then
--		local ac = select( '#', ... )
--
--		if CanIMogIt:IsTransmogable(itemID) then
--			return true
--		else
--			return false
--		end
--	else
--		--ArkInventory.Output("CanIMogIt is required to use TMog rule." )
--		return false
--	end
	-- Always return false in the end
	return false
end

function ArkInventoryRules_MoreRules.Collected( ... )
	-- Verify that the item string information is loaded and not nil and that it is a valid item before continuing
	local fn = "COLLECTED" -- Rule name for errors

	if select( 15, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) == nil or not ArkInventoryRules.Object.h or ArkInventoryRules.Object.class ~= "item" then
		return false
	end

	local itemID = select(2,ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) )
	if itemID == nil then return false end
	--local itemLink = 'item:' .. itemID
	local itemAppearanceID, isCollected, sourceID = ArkInventoryRules_MoreRules.GetItemAppearance(itemID)

	local quality = select(3, GetItemInfo(itemID))
	if quality == nil then return false end
	if quality <= 1 then return false end

	if itemAppearanceID == nil then return false end

	--if not ArkInventoryRules_MoreRules.CanTransmogItem(itemID) then
		--ArkInventory.Output(string.lower( select( 3, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) ) ..  " NOT TRANSMOGABLE" )
	--	return false
	--end

	--if ArkInventoryRules_MoreRules.PlayerCanCollectAppearance(itemAppearanceID) then
	--	return false
	--end

	if ArkInventoryRules_MoreRules.PlayerHasAppearance(itemAppearanceID) then
		--ArkInventory.Output(string.lower( select( 3, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) ) ..  " COLLECTED" )
		return true
	else
		--ArkInventory.Output(string.lower( select( 3, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) ) ..  " NOT COLLECTED" )
		return false
	end

-- Former dependency on CanIMogIt
--	if IsAddOnLoaded( "CanIMogIt" ) then			
--		local fn = "COLLECTED" -- Rule name for errors
--		
--		local ac = select( '#', ... )
--
--		-- Call to CanIMogIt functions to see if the item appearance has been collected by the player in any way
--		if CanIMogIt:PlayerKnowsTransmog(itemID)
--		or CanIMogIt:PlayerKnowsTransmogFromItem(itemID) then
--			--ArkInventory.Output(string.lower( select( 3, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) ) ..  " COLLECTED" )
--			return true
--		else
--			--ArkInventory.Output(string.lower( select( 3, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) ) ..  " NOT COLLECTED" )
--			return false
--		end
--	else
--		--ArkInventory.Output("CanIMogIt is required to use Collected rule." )
--		return false
--	end	

	-- Always return false in the end
	return false
end


function ArkInventoryRules_MoreRules.ClassGear( ... )
	local fn = "CLASSGEAR" -- Rule name for errors

	-- Verify that the item string information is loaded and not nil and that it is a valid item before continuing
	if select( 15, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) == nil or not ArkInventoryRules.Object.h or ArkInventoryRules.Object.class ~= "item" then
		return false
	end
	
	local ac = select( '#', ... )

	-- Get item information for if checks
	local class = ArkInventory.Global.Me.data.info.class
	local objtype = 	select( 15, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) )  
	local objsubtype = 	string.upper (select( 16, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) )
	local objequip = 	select( 11, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) )
	--ArkInventory.Output(objequip .. objtype .. objsubtype .. class)
	if objequip == nil then return false end
	if objsubtype == nil then return false end
	if objtype == nil then return false end 
	
	if objtype == "TRINKET" then return true end
	
	if objtype == "Armor" then
		if objsubtype == "Miscellaneous" then return true end
		if objequip == "INVTYPE_CLOAK" then return true end
		if objequip == "INVTYPE_TRINKET" then return true end
		if objequip == "INVTYPE_NECK" then return true end
		if objequip == "INVTYPE_FINGER" then return true end
		
		-- Class armor types
		if ( ( class == "DEATHKNIGHT" ) and ( objsubtype == "Plate") ) then return true end
		if ( ( class == "DEMONHUNTER" ) and ( objsubtype == "Leather") ) then return true end
		if ( ( class == "DRUID" ) and ( objsubtype == "Leather") ) then return true end
		if ( ( class == "HUNTER" ) and ( objsubtype == "Mail") ) then return true end
		if ( ( class == "MAGE" ) and ( objsubtype == "Cloth") ) then return true end
		if ( ( class == "MONK" ) and ( objsubtype == "Leather") ) then return true end
		if ( ( class == "PALADIN" ) and ( objsubtype == "Plate") ) then return true end
		if ( ( class == "PALADIN" ) and ( objsubtype == "Shields") ) then return true end
		if ( ( class == "PRIEST" ) and ( objsubtype == "Cloth") ) then return true end
		if ( ( class == "ROGUE" ) and ( objsubtype == "Leather") ) then return true end
		if ( ( class == "SHAMAN" ) and ( objsubtype == "Mail") ) then return true end
		if ( ( class == "SHAMAN" ) and ( objsubtype == "Shields") ) then return true end
		if ( ( class == "WARLOCK" ) and ( objsubtype == "Cloth") ) then return true end
		if ( ( class == "WARRIOR" ) and ( objsubtype == "Plate") ) then return true end
		if ( ( class == "WARRIOR" ) and ( objsubtype == "Shields") ) then return true end
	end
	
	-- Class weapon types
	if objtype == "Weapon" then
		
		if ( class == "DEATHKNIGHT" ) and ( 
			( objsubtype == "ONE-HANDED SWORDS") or
			( objsubtype == "TWO-HANDED SWORDS") or
			( objsubtype == "ONE-HANDED AXES") or
			( objsubtype == "TWO-HANDED AXES") or
			( objsubtype == "ONE-HANDED MACES") or
			( objsubtype == "TWO-HANDED MACES") or
			( objsubtype == "POLEARMS")
		) then return true end

		if ( class == "DEMONHUNTER" ) and ( 
			( objsubtype == "WARGLAIVES") or
			( objsubtype == "ONE-HANDED SWORDS") or
			( objsubtype == "ONE-HANDED AXES") or
			( objsubtype == "ONE-HANDED MACES") or
			( objsubtype == "DAGGERS") or
			( objsubtype == "FIST WEAPONS")
		) then return true end
		
		if ( class == "DRUID" ) and (
			( objsubtype == "STAVES") or
			( objsubtype == "ONE-HANDED MACES") or
			( objsubtype == "TWO-HANDED MACES") or
			( objsubtype == "POLEARMS") or
			( objsubtype == "DAGGERS") or
			( objsubtype == "FIST WEAPONS")
		) then return true end
	
		if ( class == "HUNTER" ) and ( 
			( objsubtype == "CROSSBOWS") or
			( objsubtype == "BOWS") or
			( objsubtype == "GUNS") or
			( objsubtype == "POLEARMS") or
			( objsubtype == "STAVES") or
			( objsubtype == "ONE-HANDED SWORDS") or
			( objsubtype == "TWO-HANDED SWORDS") or
			( objsubtype == "ONE-HANDED AXES") or
			( objsubtype == "TWO-HANDED AXES") or
			( objsubtype == "DAGGERS") or
			( objsubtype == "FIST WEAPONS")
		) then return true end
		
		if ( class == "MAGE" ) and ( 
			( objsubtype == "ONE-HANDED SWORDS") or
			( objsubtype == "STAVES") or
			( objsubtype == "DAGGERS") or
			( objsubtype == "WANDS")
		) then return true end
		
		if ( class == "MONK" ) and (
			( objsubtype == "ONE-HANDED AXES") or
			( objsubtype == "ONE-HANDED SWORDS") or
			( objsubtype == "ONE-HANDED MACES") or
			( objsubtype == "POLEARMS") or
			( objsubtype == "STAVES") or
			( objsubtype == "FIST WEAPONS")
		) then return true end
		
		if ( class == "PALADIN" ) and ( 
			( objsubtype == "ONE-HANDED SWORDS") or
			( objsubtype == "TWO-HANDED SWORDS") or
			( objsubtype == "ONE-HANDED AXES") or
			( objsubtype == "TWO-HANDED AXES") or
			( objsubtype == "ONE-HANDED MACES") or
			( objsubtype == "TWO-HANDED MACES") or
			( objsubtype == "POLEARMS")
		) then return true end
		
		if ( class == "PRIEST" ) and ( 
			( objsubtype == "STAVES") or
			( objsubtype == "WANDS") or
			( objsubtype == "DAGGERS") or
			( objsubtype == "ONE-HANDED MACES")
		) then return true end
		
		if ( class == "ROGUE" ) and (
			( objsubtype == "CROSSBOWS") or
			( objsubtype == "BOWS") or
			( objsubtype == "GUNS") or
			( objsubtype == "DAGGERS") or
			( objsubtype == "ONE-HANDED SWORDS") or
			( objsubtype == "ONE-HANDED AXES") or
			( objsubtype == "ONE-HANDED MACES") or
			( objsubtype == "FIST WEAPONS")
		) then return true end

		if ( class == "SHAMAN" ) and (
			( objsubtype == "STAVES") or
			( objsubtype == "ONE-HANDED MACES") or
			( objsubtype == "TWO-HANDED MACES") or
			( objsubtype == "ONE-HANDED AXES") or
			( objsubtype == "TWO-HANDED AXES") or
			( objsubtype == "DAGGERS") or
			( objsubtype == "FIST WEAPONS")
		) then return true end
				
		if ( class == "WARLOCK" ) and (
			( objsubtype == "ONE-HANDED SWORDS") or
			( objsubtype == "STAVES") or
			( objsubtype == "DAGGERS") or
			( objsubtype == "WANDS")
		) then return true end
		
		if ( class == "WARRIOR" ) and ( 
			( objsubtype == "CROSSBOWS") or
			( objsubtype == "BOWS") or
			( objsubtype == "GUNS") or
			( objsubtype == "ONE-HANDED SWORDS") or
			( objsubtype == "TWO-HANDED SWORDS") or
			( objsubtype == "ONE-HANDED AXES") or
			( objsubtype == "TWO-HANDED AXES") or
			( objsubtype == "ONE-HANDED MACES") or
			( objsubtype == "TWO-HANDED MACES") or
			( objsubtype == "DAGGERS") or
			( objsubtype == "POLEARMS") or
			( objsubtype == "STAVES") or
			( objsubtype == "FIST WEAPONS")
		) then return true end
			
	end
	-- Always return false in the end
	return false
end

function ArkInventoryRules_MoreRules.Upgrade( ... )
	local fn = "UPGRADE" -- Rule name for errors

	-- Verify that the item string information is loaded and not nil and that it is a valid item before continuing
	if select( 15, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) == nil or not ArkInventoryRules.Object.h or ArkInventoryRules.Object.class ~= "item" then
		return false
	end

	local ac = select( '#', ... )
	local itemID = select(2,ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) )
	if itemID == nil then return false end
	local name, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(itemID) 
    local _, _, _, slotName = GetItemInfoInstant(itemID)
    local slot = InventorySlots[slotName]
   	local ilvl = select(6,ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) )
    --ArkInventory.Output('SLOT ' .. name .. ' ' .. slotName .. ' ' .. iLevel )

    if slotName == "INVTYPE_FINGER" then
    	local finger0ilvl = select(6,ArkInventory.ObjectInfo(GetInventoryItemLink("player", 11)))
    	local finger1ilvl = select(6,ArkInventory.ObjectInfo(GetInventoryItemLink("player", 12)))
    	
    	if finger0ilvl == nil or finger1ilvl == nil then return true end
    	--ArkInventory.Output('FINGER ' .. finger0ilvl )
    	--ArkInventory.Output('FINGER ' .. name .. ' ' .. slotName .. ' ' .. ilvl )
    	
    	if ilvl >= finger0ilvl then return true end
    	if ilvl >= finger1ilvl then	return true end
    	
    	return false
    end

    if slotName == "INVTYPE_TRINKET" then
    	local trinket0ilvl = select(6,ArkInventory.ObjectInfo(GetInventoryItemLink("player", 13)))
    	local trinket1ilvl = select(6,ArkInventory.ObjectInfo(GetInventoryItemLink("player", 14)))
    	
    	if trinket0ilvl == nil or trinket1ilvl == nil then return true end
    	--ArkInventory.Output('TRINKET ' .. trinket0ilvl )
    	--ArkInventory.Output('TRINKET' .. name .. ' ' .. slotName .. ' ' .. ilvl )
    	
    	if ilvl >= trinket0ilvl then return true end
    	if ilvl >= trinket1ilvl then return true end
    	
    	return false
    end

	local objtype = select( 15, ArkInventory.ObjectInfo( ArkInventoryRules.Object.h ) ) 
	if not objtype == "ARMOR" or not objtype == "WEAPON" then return false end

	if slotName == nil or slot == nil then 
		return false
	else
		-- Take into account 2 handed weapons or ranged weapons when comparing offhand gear
		local eqSlotName = select(11,ArkInventory.ObjectInfo(GetInventoryItemLink("player", 16)))
		if eqSlotName == "INVTYPE_2HWEAPON" or eqSlotName == "INVTYPE_RANGED" then
			if slot == 17 then slot = 16 end
		end
		
		local eqilvl = select(6,ArkInventory.ObjectInfo(GetInventoryItemLink("player", slot)))
		

		if eqilvl == nil then return true end
		if ilvl >= eqilvl then return true end
		--ArkInventory.Output('wearingID' .. name .. wearingID )
		--ArkInventory.Output('slotname' .. name .. slotName )
		--ArkInventory.Output('slot' .. name .. slot )
		--if objequip == "INVTYPE_HEAD" or
	   	--objequip == "" or
	   	--objequip == "" or
		--objequip == "" or
		--then
		--end
	-- Always return false in the end
	end
	return false
end
